"use strict";
var io = require("socket.io-client");
var Vue = require("vue");
var vue_class_component_1 = require("vue-class-component");
var Clipboard = require("clipboard");
var browser_1 = require("js-split-file/browser");
var supportWebRTC = !!window.RTCPeerConnection;
function getRoom() {
    return Math.round(Math.random() * 35 * Math.pow(36, 9)).toString(36);
}
function getNow() {
    var now = new Date();
    var hours = now.getHours();
    var minutes = now.getMinutes();
    var seconds = now.getSeconds();
    return (hours < 10 ? "0" + hours : hours) + ":" + (minutes < 10 ? "0" + minutes : minutes) + ":" + (seconds < 10 ? "0" + seconds : seconds);
}
function drawQRCode() {
    new QRCodeLib.QRCodeDraw().draw(document.getElementById("qr"), document.location.href, function (error) {
        if (error) {
            console.log(error);
        }
    });
}
new Clipboard(".clipboard");
var worker = new Worker("worker.bundle.js");
var App = (function (_super) {
    __extends(App, _super);
    function App(options) {
        var _this = _super.call(this) || this;
        _this.acceptMessages = [];
        _this.newText = "";
        _this.id = 1;
        _this.locale = navigator.language;
        _this.clientCount = 0;
        _this.peerConnection = supportWebRTC ? new RTCPeerConnection() : null;
        _this.dataChannelIsOpen = false;
        _this.dataChannel = null;
        _this.splitFile = new browser_1.SplitFile();
        _this.files = [];
        _this.speed = 100;
        var hash = document.location.hash;
        var room;
        if (!hash || hash === "#") {
            room = getRoom();
            document.location.hash = "#" + room;
        }
        else {
            room = hash.substr(1);
        }
        var connect = function () {
            _this.socket = io("/", { query: { room: room } });
            _this.socket.on("copy", _this.onMessageRecieved);
            _this.socket.on("message_sent", _this.onMessageSent);
            _this.socket.on("client_count", _this.onClientCount);
            if (supportWebRTC) {
                _this.socket.on("offer", _this.onGetOffer);
                _this.socket.on("answer", _this.onGetAnswer);
            }
            drawQRCode();
        };
        connect();
        window.onhashchange = (function (e) {
            if (e.newURL) {
                var index = e.newURL.indexOf("#");
                if (index > -1) {
                    var newRoom = e.newURL.substring(index + 1);
                    if (room !== newRoom) {
                        _this.socket.disconnect();
                        room = newRoom;
                        connect();
                    }
                }
            }
        });
        if (_this.peerConnection) {
            _this.dataChannel = _this.peerConnection.createDataChannel("copy_tool_channel_name");
            _this.peerConnection.ondatachannel = function (event) {
                event.channel.onopen = function (e) {
                    app.dataChannelIsOpen = true;
                    _this.acceptMessages.unshift({
                        kind: "text",
                        value: "The connection is opened.",
                        moment: getNow(),
                        id: _this.id++,
                    });
                };
                event.channel.onclose = function (e) {
                    app.dataChannelIsOpen = false;
                    _this.acceptMessages.unshift({
                        kind: "text",
                        value: "The connection is closed.",
                        moment: getNow(),
                        id: _this.id++,
                    });
                };
                event.channel.onmessage = function (e) {
                    if (typeof e.data === "string") {
                        _this.acceptMessages.unshift({
                            kind: "text",
                            value: e.data,
                            moment: getNow(),
                            id: _this.id++,
                        });
                    }
                    else {
                        var block_1 = _this.splitFile.decodeBlock(new Uint8Array(e.data));
                        var currentBlockIndex = _this.files.findIndex(function (f) { return f.fileName === block_1.fileName; });
                        if (currentBlockIndex === -1) {
                            currentBlockIndex = _this.files.length;
                            _this.files.push({
                                fileName: block_1.fileName,
                                blocks: [],
                                progress: 0,
                            });
                        }
                        var currentBlock = _this.files[currentBlockIndex];
                        currentBlock.blocks.push({
                            currentBlockIndex: block_1.currentBlockIndex,
                            binary: block_1.binary,
                        });
                        currentBlock.progress = Math.round(currentBlock.blocks.length * 100.0 / block_1.totalBlockCount);
                        if (currentBlock.blocks.length === block_1.totalBlockCount) {
                            currentBlock.blocks.sort(function (a, b) { return a.currentBlockIndex - b.currentBlockIndex; });
                            var file = new File(currentBlock.blocks.map(function (f) { return f.binary; }), block_1.fileName);
                            _this.acceptMessages.unshift({
                                kind: "file",
                                value: file,
                                url: URL.createObjectURL(file),
                                moment: getNow(),
                                id: _this.id++,
                            });
                            _this.files.splice(currentBlockIndex, 1);
                        }
                    }
                };
            };
        }
        return _this;
    }
    Object.defineProperty(App.prototype, "canCreateOffer", {
        get: function () {
            return supportWebRTC && !this.dataChannelIsOpen;
        },
        enumerable: true,
        configurable: true
    });
    App.prototype.onGetAnswer = function (data) {
        var answer = new RTCSessionDescription(data.answer);
        this.peerConnection.setRemoteDescription(answer);
    };
    App.prototype.onGetOffer = function (data) {
        var _this = this;
        var offer = new RTCSessionDescription(data.offer);
        this.peerConnection.setRemoteDescription(offer)
            .then(function () { return _this.peerConnection.createAnswer(); })
            .then(function (answer) { return _this.peerConnection.setLocalDescription(answer); })
            .then(function () {
            _this.socket.emit("answer", {
                sid: data.sid,
                answer: _this.peerConnection.localDescription.toJSON(),
            });
        });
    };
    App.prototype.tryToConnect = function () {
        var _this = this;
        if (this.peerConnection) {
            this.peerConnection.createOffer()
                .then(function (offer) { return _this.peerConnection.setLocalDescription(offer); })
                .then(function () {
                _this.socket.emit("offer", _this.peerConnection.localDescription.toJSON());
            });
        }
    };
    App.prototype.onMessageRecieved = function (data) {
        if (data.kind === "file") {
            var file = new File([data.value], data.name, { type: data.type });
            this.acceptMessages.unshift({
                kind: "file",
                value: file,
                url: URL.createObjectURL(file),
                moment: getNow(),
                id: this.id++,
            });
        }
        else {
            data.moment = getNow();
            data.id = this.id++;
            this.acceptMessages.unshift(data);
        }
    };
    App.prototype.onMessageSent = function (data) {
        this.acceptMessages.unshift({
            kind: "text",
            value: "the " + data.kind + " is sent successfully to " + this.clientCount + " clients.",
            moment: getNow(),
            id: this.id++,
        });
    };
    App.prototype.onClientCount = function (data) {
        this.clientCount = data.clientCount;
    };
    Object.defineProperty(App.prototype, "buttonText", {
        get: function () {
            if (this.clientCount > 0) {
                return "Copy the text to " + this.clientCount + " clients";
            }
            return "No clients to sent";
        },
        enumerable: true,
        configurable: true
    });
    App.prototype.copyText = function () {
        if (this.clientCount <= 0) {
            this.acceptMessages.unshift({
                kind: "text",
                value: "No clients to sent.",
                moment: getNow(),
                id: this.id++,
            });
            return;
        }
        if (!this.newText) {
            this.acceptMessages.unshift({
                kind: "text",
                value: "No text to sent.",
                moment: getNow(),
                id: this.id++,
            });
            return;
        }
        if (this.dataChannelIsOpen) {
            this.dataChannel.send(this.newText);
        }
        else {
            var copyData = {
                kind: "text",
                value: this.newText,
            };
            this.socket.emit("copy", copyData);
        }
        this.newText = "";
    };
    App.prototype.fileUploaded = function (file) {
        if (this.clientCount <= 0) {
            this.acceptMessages.unshift({
                kind: "text",
                value: "No clients to sent.",
                moment: getNow(),
                id: this.id++,
            });
            return;
        }
        var extensionName = file.type.split("/")[1];
        var fileName = file.name || "no name." + extensionName;
        if (this.dataChannelIsOpen) {
            var message = {
                kind: "split file",
                file: file,
                fileName: fileName,
            };
            worker.postMessage(message);
        }
        else {
            if (file.size >= 10 * 1024 * 1024) {
                this.acceptMessages.unshift({
                    kind: "text",
                    value: "the file is too large(>= 10MB).",
                    moment: getNow(),
                    id: this.id++,
                });
                return;
            }
            this.socket.emit("copy", {
                kind: "file",
                value: file,
                name: fileName,
                type: file.type,
            });
        }
    };
    return App;
}(Vue));
App = __decorate([
    vue_class_component_1.default({
        template: require("raw!./app.html"),
    })
], App);
var app = new App({
    el: "#body",
});
var blocks = [];
setInterval(function () {
    if (blocks.length > 0) {
        blocks.splice(0, app.speed).forEach(function (block) {
            app.dataChannel.send(block);
        });
    }
}, 1000);
worker.onmessage = function (e) {
    var message = e.data;
    if (message.kind === "split file result") {
        blocks.push.apply(blocks, message.blocks);
    }
};
//# sourceMappingURL=index.js.map